from nseazy._helpers import _headers, _cookies
import requests


method = True

if(method is True):
    def _fetch_data(api_url):
        session = requests.Session()
        rawData = session.get(api_url,headers=_headers,cookies=_cookies).json()
        return rawData


import pandas as pd
import textwrap
import nseazy as nz

def df_wrapcols(df,wrap_columns=None):

    if wrap_columns is None: return df
    if not isinstance(wrap_columns,dict):
        raise TypeError('wrap_columns must be a dict of column_names and wrap_lengths')

    for col in wrap_columns:
        if col not in df.columns:
            raise ValueError('column "'+str(col)+'" not found in df.columns')

    index = []
    column_data = {}
    for col in df.columns:
        column_data[col] = []
  
    for ix in df.index:
        row = df.loc[ix,]
        
        row_data = {}
        for col in row.index:
            cstr = str(row[col])
            if col in wrap_columns:
                wlen = wrap_columns[col]
                tw   = textwrap.wrap(cstr,wlen) if not cstr.isspace() else [' ']
            else:
                tw = [cstr]
            row_data[col] = tw

        cmax = max(row_data,key=lambda k: len(row_data[k]))
        rlen = len(row_data[cmax])
        for r in range(rlen):
            for col in row.index:
                extension = [' ']*(rlen - len(row_data[col]))
                row_data[col].extend(extension)
                column_data[col].append(row_data[col][r])
            ixstr = str(ix)+'.'+str(r) if r > 0 else str(ix)
            index.append(ixstr)

    return pd.DataFrame(column_data,index=index)

def make_left_formatter(maxwidth):
    wm3 = maxwidth-3
    w   = maxwidth
    def left_formatter(value):
        if not isinstance(value,str):
            return f'{value:<}'
        elif value[0:maxwidth] == '-'*maxwidth:
            return f'{value:<{w}.{w}s}'
        elif len(value) > maxwidth:
            return f'{value:<{wm3}.{wm3}s}...'
        else:
            return f'{value:<{w}.{w}s}'
    return left_formatter

def help( func_name=None, kwarg_names=None, sort=False ):

    func_kwarg_map = {
        'show_data'         : nz.fetch._get_quote_parameter,
        }
    func_kwarg_aliases = {
        'addplot'           : nz.fetch._get_quote_parameter,
        }
    
    
    if func_name is None:
        print('\nUsage: `kwarg_help(func_name)` or `kwarg_help(func_name,kwarg_names)`')
        print('        kwarg_help is available for the following func_names:')
        s = str(list(func_kwarg_map.keys()))
        text = textwrap.wrap(s,68)
        for t in text:
            print('           ',t)
        print()
        return

    fkmap = {**func_kwarg_map, **func_kwarg_aliases}
           
    if func_name not in fkmap:
        raise ValueError('Function name "'+func_name+'" NOT a valid function name')

    if kwarg_names is not None and isinstance(kwarg_names,str):
        kwarg_names = [ kwarg_names, ]

    if ( kwarg_names is not None 
         and (not isinstance(kwarg_names,(list,tuple)) 
              or not all([isinstance(k,str) for k in kwarg_names])
             ) 
       ):
        raise ValueError('kwarg_names must be a sequence (list,tuple) of strings')

    vks = fkmap[func_name]()

    df = (pd.DataFrame(vks).T).drop('Validator',axis=1)
    df.index.name = 'Kwarg'
    if sort: df.sort_index(inplace=True)
    df.reset_index(inplace=True)

    if kwarg_names is not None:
        for k in kwarg_names:
            if k not in df['Kwarg'].values:
                print('   Warning: "'+k+'" is not a valid `kwarg_name` for `func_name` "'+func_name,'"')
        df = df[ df['Kwarg'].isin(kwarg_names) ]
        if len(df) < 1:
            raise ValueError(' None of specified `kwarg_names` are valid for `func_name` "'+func_name,'"')

    df['Default'] = ["'"+d+"'" if isinstance(d,str) else str(d) for d in df['Default']]
        
    klen = df['Kwarg'].str.len().max()+1
    dlen = df['Default'].str.len().max()+1

    wraplen = max( 40, 80-(klen+dlen) )
    df = df_wrapcols(df,wrap_columns={'Description':wraplen})

    dividers = []
    for col in df.columns:
        dividers.append('-'*int(df[col].str.len().max()))
    dfd = pd.DataFrame(dividers).T
    dfd.columns = df.columns
    dfd.index = pd.Index(['---'])

    df = pd.concat([dfd,df])

    formatters = { 'Kwarg'       : make_left_formatter( klen ),
                   'Default'     : make_left_formatter( dlen ),
                   'Description' : make_left_formatter( wraplen ),
                 }
    
    print('\n ','-'*78)
    print('  Following Parameter Can Be Used at "'+func_name+'":')

    s = df.to_string(formatters=formatters,index=False,justify='left')

    print('\n ',s.replace('\n','\n  '))


from nseazy._generate_request import _fetch_data

def _validate_symbol(symbol):
    symbol = symbol.translate({ord('-'): None,ord('&'): None, }).upper()

    if any(x in symbol for x in fnolist):
        isDerivative = True
    else:
        isDerivative = False
    
    return symbol , isDerivative


def _instrument_derivative():
    positions = _fetch_data('https://www.nseindia.com/api/equity-stockIndices?index=SECURITIES%20IN%20F%26O')

    nselist=['NIFTY','NIFTYIT','BANKNIFTY']

    i=0
    for x in range(i, len(positions['data'])):
        nselist=nselist+[positions['data'][x]['symbol']]

    return nselist

fnolist = _instrument_derivative()


import requests
import time

_headers = {
    'Connection': 'keep-alive',
    'Cache-Control': 'max-age=0',
    'DNT': '1',
    'Upgrade-Insecure-Requests': '1',
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.79 Safari/537.36',
    'Sec-Fetch-User': '?1',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
    'Sec-Fetch-Site': 'none',
    'Sec-Fetch-Mode': 'navigate',
    'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Language': 'en-US,en;q=0.9,hi;q=0.8',
    
}

def get_cookies(_base_api_url):
    session = requests.Session()
    request = session.get(_base_api_url,headers=_headers)
    if request.status_code == 200:
        cookies = dict(request.cookies)
    else:
        time.sleep(5)
        request = session.get(_base_api_url,headers=_headers)
        cookies = dict(request.cookies)
    return cookies


_base_api_url = "https://www.nseindia.com/"

_equity_quote_api = "api/quote-equity?symbol="

_derivative_quote_api = 'api/quote-derivative?symbol='

_option_chain_api = 'api/option-chain-indices?symbol='

_holidays_api = 'api/holiday-master?type='

_cookies = get_cookies(_base_api_url)





import nseazy._warnings
from nseazy._version    import __version__
from nseazy._help import help
from nseazy._helpers import _cookies
from nseazy._instruments import fnolist
from nseazy.fetch import *




def _validate_vkwargs_dict(vkwargs):

    for key, value in vkwargs.items():
        
        if len(value) != 3:
            raise ValueError('Items != 3 in valid kwarg table, for kwarg "'+key+'"')
        if 'Default' not in value:
            raise ValueError('Missing "Default" value for kwarg "'+key+'"')
        if 'Description' not in value:
            raise ValueError('Missing "Description" value for kwarg "'+key+'"')
        if 'Validator' not in value:
            raise ValueError('Missing "Validator" function for kwarg "'+key+'"')
        
def _check_kwargs(kwargs, vkwargs):

    config  = {}
    for key, value in vkwargs.items():
        config[key] = value['Default']

    for key in kwargs.keys():
        if key not in vkwargs:
            raise KeyError('Unrecognized kwarg="'+str(key)+'"')
        else:
            value = kwargs[key]
            try:
                valid = vkwargs[key]['Validator'](value)
            except Exception as ex:
                ex.extra_info = 'kwarg "'+key+'" validator raised exception to value: "'+str(value)+'"'
                raise
            if not valid:
                import inspect
                v = inspect.getsource(vkwargs[key]['Validator']).strip()
                raise TypeError('kwarg "'+key+'" validator returned False for value: "'+str(value)+'"\n    '+v)


        config[key] = value

    return config


version_info = (0, 0, 1, 'beta', 2)

_specifier_ = {'alpha': 'a','beta': 'b','candidate': 'rc','final': ''}

__version__ = '%s.%s.%s%s'%(version_info[0], version_info[1], version_info[2],
  '' if version_info[3]=='final' else _specifier_[version_info[3]]+str(version_info[4]))


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(name='nseazy',
      version='0.0.1b5',
      author='Dr Chandrakant',
      author_email='nseazy-users@python.org',
      maintainer_email='DrChandrakant.github@gmail.com',
      description='Utilities for the NSE India API',
      long_description=long_description,
      long_description_content_type='text/markdown; charset=UTF-8',
      url='https://github.com/DrChandrakant/NSEazy',
      platforms='Cross platform (Linux, Mac OSX, Windows)',
      install_requires=['pandas','requests'],
      license="BSD-style",
      packages=setuptools.find_packages(),
      classifiers=['Development Status :: 4 - Beta',
                   'Programming Language :: Python :: 3',
                   'Programming Language :: Python :: 3.6',
                   'Programming Language :: Python :: 3.7',
                   'Programming Language :: Python :: 3.8',
                   'Programming Language :: Python :: 3.9',
                   'Intended Audience :: Developers',
                   'Intended Audience :: Education',
                   'Intended Audience :: Financial and Insurance Industry',
                   'Intended Audience :: Science/Research',
                   'License :: OSI Approved :: BSD License',
                   'Topic :: Office/Business :: Financial',
                   'Topic :: Office/Business :: Financial :: Investment',
                   'Topic :: Scientific/Engineering :: Visualization',
                   'Topic :: Scientific/Engineering :: Information Analysis',
                   ],
      keywords=['finance','candlestick','ohlc','market','investing','technical analysis'],
      )



import pandas as pd
from nseazy._helpers import _base_api_url, _equity_quote_api, _derivative_quote_api, _holidays_api
from nseazy._instruments import _validate_symbol
from nseazy._generate_request import _fetch_data
from nseazy._validators import _validate_vkwargs_dict, _check_kwargs

def nse_quote(symbol,section=""):
    symbol, isDerivative = _validate_symbol(symbol)

    if(section==""):
        if isDerivative is True:
            rawJson = _fetch_data( _base_api_url + _derivative_quote_api + symbol)
        else:
            rawJson = _fetch_data( _base_api_url + _equity_quote_api + symbol)
        return rawJson

    if(section!=""):
        rawJson = _fetch_data( _base_api_url + _equity_quote_api + symbol+'&section='+section)
        return rawJson
    
def nse_optionchain_scrapper(symbol):
    symbol, isDerivative = _validate_symbol(symbol)
    if isDerivative is True:
        rawOptionchain = _fetch_data( _base_api_url + _derivative_quote_api + symbol)
    else:
        rawOptionchain = None
    return rawOptionchain

def nse_fno(symbol):
    symbol, isDerivative = _validate_symbol(symbol)
    try:
        rawJson = _fetch_data( _base_api_url + _derivative_quote_api + symbol)
        try:
            if(len(rawJson['stocks'])== 0):
                print(symbol,'May Not Be List In Derivative Segemnt. Trying Fetch Data From Equity Segment')
                rawJson = _fetch_data( _base_api_url + _equity_quote_api + symbol)
        except KeyError:
            pass
    except KeyError:
        print("Getting Error While Fetching.")
    return rawJson

def nse_eq(symbol):
    symbol, isDerivative = _validate_symbol(symbol)
    try:
        rawJson = _fetch_data( _base_api_url + _equity_quote_api + symbol)
        try:
            if(len(rawJson['stocks'])== 0):
                print(symbol,'May Not Be List In Equity Segemnt. Trying Fetch Data From Equity Segment')
                rawJson = _fetch_data( _base_api_url + _derivative_quote_api + symbol)
        except KeyError:
            pass
    except KeyError:
        print("Getting Error While Fetching.")
    return rawJson

def quote_equity(symbol):
    return nse_eq(symbol)

def quote_derivative(symbol):
    return nse_fno(symbol)

def option_chain(symbol):
    return nse_optionchain_scrapper(symbol)

def nse_holidays(type="trading"):
    if(type=="clearing"):
        rawHoliday = _fetch_data(_base_api_url+_holidays_api + type)
    if(type=="trading"):
        rawHoliday = _fetch_data(_base_api_url+_holidays_api + type)
    return rawHoliday



def _get_quote_parameter():
    vkwargs = {
        'Future'    : { 'Default'       : False,
                          'Description' : "If Derivative Segment Data Required. Set True ",
                          'Validator'   : lambda value: isinstance(value,bool) },
        'Info'      : { 'Default'       : False,
                          'Description' : "If Company Information Required. Set True ",
                          'Validator'   : lambda value: isinstance(value,bool) },
        'LTP'       : { 'Default'       : True,
                          'Description' : "If Last Traded Price(LTP) Required. Set True ",
                          'Validator'   : lambda value: isinstance(value,bool) },
        'OHLCV'     : { 'Default'       : False,
                          'Description' : "If Open High Low Close and Volumne Data Required. Set True ",
                          'Validator'   : lambda value: isinstance(value,bool) },
        'Pre'       : { 'Default'       : False,
                          'Description' : "If Pre Open Market Data Required. Set True ",
                          'Validator'   : lambda value: isinstance(value,bool) },     
    }
    _validate_vkwargs_dict(vkwargs)
    return vkwargs

def show_data(symbol,kwargs):

    symbol, isDerivative = _validate_symbol(symbol)

    config = _check_kwargs(kwargs, _get_quote_parameter())


    Future = config['Future']
    Info   = config['Info']
    OHLCV  = config['OHLCV']
    Popen  = config['Pre']



    if Future == True:
        data = quote_derivative(symbol)
        print(data)
    
    if Future == False:
        data = quote_equity(symbol)
        company_info = {
            'Name'          : data['info']['companyName'],
            'Symbol'        : data['info']['symbol'],
            'Sector'        : data['info']['industry'],
            'ISIN'          : data['metadata']['isin'],
            'Sector PE'     : data['metadata']['pdSectorPe'],
            'PE'            : data['metadata']['pdSymbolPe'],
            'Index'         : data['metadata']['pdSectorInd'],
            'Face Value'    : data['securityInfo']['faceValue'],
            'Total Equity'  : data['securityInfo']['issuedSize']
        }
        olhc = {
            'Open'         : data['priceInfo']['open'],
            'LTP'          : data['priceInfo']['lastPrice'],
            'Change'       : data['priceInfo']['change'],
            '% Chg'        : round(data['priceInfo']['pChange'],2),
            'Final Close'  : data['priceInfo']['close'],
            'Previous'     : data['priceInfo']['previousClose'],
            'VWAP'         : data['priceInfo']['vwap'],
            'Lower Circuit': data['priceInfo']['lowerCP'],
            'Upper Circuit': data['priceInfo']['upperCP'],
        }

        df_info = pd.DataFrame([company_info])
        df_olhc = pd.DataFrame([olhc])
        ltp =  data['priceInfo']['lastPrice']
    if Info == True:
        print(df_info)
    if OHLCV == True:
        print(df_olhc)
    if config['LTP'] == True:
        print(ltp)


